#!/bin/bash

# Project "fingerprintGUI": Services for fingerprint authentication on Linux
# Module: configure.sh
# Purpose: script to create a Makefile for 32bit or 64bit system
#
# @author  Wolfgang Ullrich
# Copyright (C) 2008-2009  Wolfgang Ullrich

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# Use this script to create a Makefile needed to compile.
# After Makefile is created call "make".

# First we remove all older makefiles
find . | grep Makefile | grep -v ".mk" | xargs rm -f

# Test whether "qmake" is available
QMAKE=`which qmake`

if [ "$QMAKE" == "" ]; then
    echo
    echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    echo "Could not find \"qmake\". Did you install the Qt development system?"
    echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    exit 0
fi

# Test whether we are running a 32bit or a 64bit system
if [ "`uname -a | grep x86_64`" == "" ] ; then
    LINUX="32"
else
    LINUX="64"
fi

# Create project files
DIRLIST="bin/fingerprintGUI bin/fingerprintHelper bin/fingerprintIdentifier bin/fingerprintPAM bin/fingerprintPlugin"
for DIR in $DIRLIST ; do
    TARGET=`echo $DIR | sed s/"bin\/"//`
    cp $DIR/pro.$LINUX $DIR/$TARGET.pro
    if [ -d $DIR/fprint-only ]; then
	TARGET="fprint-only"
	cp $DIR/fprint-only/pro.$LINUX $DIR/fprint-only/fprint-only.pro 
    fi
done

# And now create a new makefile
$QMAKE

echo Makefile created. You can call \"make\" now.

