/*
 * Project "fingerprintGUI": Services for fingerprint authentication on Linux
 * Module: FingerprintDevice.cpp, FingerprintDevice.h
 * Purpose: An USB device to deal with a fingerprint sensor
 *
 * @author  Wolfgang Ullrich
 * Copyright (C) 2008-2009  Wolfgang Ullrich
 */

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _FINGERPRINTDEVICE_H
#define _FINGERPRINTDEVICE_H

#define _MULTI_THREADED

#include <string>
#include <QObject>
#include "UsbDevice.h"

#include "globals.h"

using namespace std;

//This struct was copied from libfprint "fp_internal.h" an is used only for 
//getting the vendorID and productID of the device to build the display name from.
//This might lead to weird results if the struct is changed in future versions
//of libfprint. It had have been a wise decision to provide a getter function for 
//vendorID and productID of the discovered device in libfprint because the 
//drivername is not sufficient to indentify a device (several devices might use
//the same driver).
struct fp_dscv_dev {
    struct usb_device *udev;
    struct fp_driver *drv;
    unsigned long driver_data;
    uint32_t devtype;
};

//typedef enum {
enum {
    MODE_ACQUIRE=10,
    MODE_VERIFY,
    MODE_IDENTIFY,
};

//FingerprintDevice
class FingerprintDevice : public QThread
 {
    Q_OBJECT

    string displayName;
    string driverName;
    struct fp_dscv_dev *fpDevice;
    struct fp_dev *dev;
    string bsDevice;
    struct fp_print_data *fpData;
    bool isFp;
    int mode;
    void **identifyData;
    int numIdentify;
#ifdef HAS_LIBBSAPI
    ABS_BIR* bsData;
    ABS_CONNECTION conn;
    int absFlag;
#endif

public:
    int vendorId;
    int productId;
    FingerprintDevice* next;

public:
    FingerprintDevice(struct fp_dscv_dev *fp,USBDevice *knownUSBDevices=NULL);
#ifdef HAS_LIBBSAPI
    FingerprintDevice(struct abs_device_list_item *bs,USBDevice *knownUSBDevices=NULL);
    void emitAcquireResult(int result);
    void emitVerifyResult(int result);
#endif
    string* getDisplayName(int);            //returns a display name for this device
    bool canIdentify();
    struct fp_dscv_dev *getFpDevice();      //returns the fp_dscv_dev to be used
    void setData(void *data,int size);      //loads data into driver
    int getData(void **d);                  //gets data, return datasize
    bool isFpDevice();
    void setMode(int mode);
    void setTimeout(bool timeout);          //timeot after OPERATION_TIMEOUT if true
    void setIdentifyData(void **identifyData,int numIdentify);
    void stop();

protected:
    void run();
private :
    bool acquire();
    bool verify();
    bool identify();

public slots:
signals:
    void neededStages(int stages);
    void acquireResult(int result);
    void verifyResult(int result);
    void matchResult(int match);
    void noDeviceOpen();
};

#endif /* _FINGERPRINTDEVICE_H */
