/*
 * Project "fingerprintGUI": Services for fingerprint authentication on Linux
 * Module: PamGUI.cpp, PamGUI.h
 * Purpose: Main object for fingerprintPAM module for running in gui environments
 *
 * @author  Wolfgang Ullrich
 * Copyright (C) 2008-2009  Wolfgang Ullrich
 */

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _PAMGUI_H
#define	_PAMGUI_H

#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <string>
#include <QtGui>
#include <QObject>
#include <QEventLoop>
#include <security/pam_modules.h>
#include "globals.h"
#include "ui_fingerprintPlugin.h"
#include "FingerprintDevice.h"
#include "FingerprintData.h"
#include "FingerprintDiscoverer.h"
#include "DeviceHandler.h"

#include "globals.h"

class PamGUI : public QFrame, public Ui::MainFrame {
    Q_OBJECT

private:
    QTimer *timer;
    int repeatDelay;
    int fifo;
    QMovie *movie;
    FingerprintDevice *device;
    void **identifyData;
    FingerprintData **fingerprintData;

public:

    //used for authentication
    PamGUI(FingerprintDevice *device,const char *user=NULL,const char *finger=NULL,QWidget *parent=0,Qt::WFlags f=0);
    //used for identification
    PamGUI(FingerprintDevice *device,void **identifyData,FingerprintData **fingerprintData,QWidget *parent=0,Qt::WFlags f=0);

private:
    void startupGUI(FingerprintDevice *dev);
    void showMessage(bool bold,const QString msg);
    
public slots:
    void timerTick();
    void matchResult(int match);
    void newVerifyResult(int result);
};

#endif	/* _PAMGUI_H */
