#!/bin/bash

# Project "fingerprintGUI": Services for fingerprint authentication on Linux
# Module: install.sh
# Purpose: Install/uninstall script
#
# @author  Wolfgang Ullrich
# Copyright (C) 2008-2009  Wolfgang Ullrich

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# Target directories
BIN=/usr/local/bin
SECURITY=/lib/security
LIBDIR="/usr/local/lib"
APPDIR="/usr/share/applications"

# Sourcefiles
GUI="fingerprintGUI"
GUI_STARTER="fingerprintGUI.desktop"
IDENT="fingerprintIdentifier"
HELPER="fingerprintHelper"
PLUGIN="fingerprintPlugin"
PAMDIR="fingerprintPAM"
PAM="libpam_fingerprint.so*"
UPEKDIR="./upek/lib"
UPEKSO="libbsapi.so"

# test whether Ruth does it
if test `id -u` != "0"; then
echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
echo !! You must be \"root\" to install the binaries !!
echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
exit -1
fi

# Test whether we are running a 32bit or a 64bit system
if [ "`uname -a | grep x86_64`" == "" ] ; then
    LINUX="32bit"
else
    LINUX="64bit"
fi

# test whether ldd is available
LDD=`which ldd`
if test -z $LDD ; then
echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
echo !!!!! Couldn\'t find \"ldd\" command in path !!!!!!
echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
exit -1
fi

for D in $BIN $SECURITY $LIBDIR $APPDIR ; do
    if ! [ -d $D ] ; then
	echo "ERROR: Missing target directory \"$D\""
	exit -1
    fi
done

case "$1" in
    "--with-upek")
	SUBDIR="with-upek"
	if [ "$LINUX" == "64bit" ]; then # We are running a 64bit system
		GETLIBS=`which getlibs`
		if [ "$GETLIBS" == "" ]; then
			echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			echo !!!!! Couldn\'t find \"getlibs\" command in path !!!!!!
			echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			exit -1
		fi
	fi
	UPEKLIB="cp -f $UPEKDIR/$UPEKSO $LIBDIR"
	;;
    "--fprint-only")
	SUBDIR="fprint-only/$LINUX"
	UPEKLIB=""
	;;
    "--uninstall")
	rm -f $BIN/$GUI
	rm -f $APPDIR/$GUI_STARTER
	rm -f $BIN/$IDENT
	rm -f $BIN/$HELPER
	rm -f $BIN/$PLUGIN
	rm -f $SECURITY/$PAM
	rm -f $LIBDIR/$UPEKSO
	echo "Fingerprint binaries have been uninstalled."
	exit 0
	;;
    *)
	echo "USAGE: ./install.sh [--with-upek | --fprint-only | --uninstall]"
	exit -1
	;;
esac

APPLIST="./bin/$GUI/$SUBDIR/$GUI ./bin/$IDENT/$SUBDIR/$IDENT ./bin/$HELPER/$SUBDIR/$HELPER"
# We need to install 32bit versions of APPLIST on 64bit systems with upek
if [ "$GETLIBS" != "" ]; then	# It's a 64bit system and --with-upek is requested
	for PROG in $APPLIST ; do
		$GETLIBS $PROG
	done 
fi
# Append the "PAM" and "plugin" module to APPLIST
APPLIST="$APPLIST ./bin/$PLUGIN/$LINUX/$PLUGIN ./bin/$PAMDIR/$LINUX/$PAM"

# check for missing libraries
MISSING=`ldd $APPLIST | grep "not found" | awk '{ print $1 }' | sort -u | grep -v $UPEKSO `
# check for libbeecrypt.so.6 (since version 0.11)
CRYPTOLIB=`find /usr/lib | grep libqca-ossl`
if test -z "$CRYPTOLIB" ; then
	MISSING="$MISSING libqca-ossl.so"
fi
if test -n "$MISSING" ; then
echo "PLEASE INSTALL MISSING SHARED LIBRARIES FIRST:"
    for M in $MISSING ; do
	echo $M
    done
    exit -1
fi

cp -f ./bin/$GUI/$SUBDIR/$GUI $BIN
cp -f ./bin/$GUI/$GUI_STARTER $APPDIR
cp -f ./bin/$IDENT/$SUBDIR/$IDENT $BIN
cp -f ./bin/$HELPER/$SUBDIR/$HELPER $BIN
cp -f ./bin/$PLUGIN/$LINUX/$PLUGIN $BIN
cp -fP ./bin/$PAMDIR/$LINUX/$PAM $SECURITY
$UPEKLIB
ldconfig

echo "Fingerprint binaries have been installed."

exit 0

