/*
 * Project "fingerprintGUI": Services for fingerprint authentication on Linux
 * Module: PamTester.cpp, PamTester.h
 * Purpose: Testing of PAM settings with "libpam_fingerprint.so"
 *
 * @author  Wolfgang Ullrich
 * Copyright (C) 2008-2009  Wolfgang Ullrich
 */

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../include/PamTester.h"

volatile bool stopTester=false;
static volatile bool returnPressed=false;
static pam_response* pamResponse=NULL; 
QApplication *app;

int callback(int,const struct pam_message **,struct pam_response **resp,void *){
    pamResponse=(pam_response*)malloc(sizeof(pam_response));
    closelog();
    openlog(syslogIdent.data(),LOG_NDELAY|LOG_PID,LOG_AUTH);
    if(debugTest)
        setlogmask(-1);
    else
        setlogmask(LOG_UPTO(LOG_ERR));
    syslog(LOG_DEBUG,"Caught PAM_CALLBACK, waiting.");
    while(!(returnPressed|stopTester)){
        app->processEvents();
        usleep(50000);
    }
    if(stopTester)  // Application will exit
        return PAM_CONV_ERR;
    pamResponse->resp_retcode=PAM_SUCCESS;
    *resp=pamResponse;
    syslog(LOG_DEBUG,"Return: PAM_SUCCESS.");
    returnPressed=false;
    return PAM_SUCCESS;
}

PamTester::PamTester(const char *svc,bool identify){
    if(svc==NULL)return; //uups
    service=svc;
    pamconv=(pam_conv){callback,NULL};
    pamh=NULL;
    user=NULL;
    app=qApp;
    if(!identify)   // This service doesn't require to identify the user
        user=getlogin();
}

void PamTester::testPam(QLineEdit *result){
    stopTester=false;
    returnPressed=false;
    pamResponse=(pam_response*)malloc(sizeof(pam_response));
    int rc;
    line=result;
    string message;

    syslog(LOG_DEBUG,"Testing service \"%s\"",service);
    line->clear();
    line->setFocus();
// Release the fingerprint device to be used by libpam_fingerprint.so
    fp_exit();//exit libfprint
#ifdef HAS_LIBBSAPI
    if(ABSTerminate()!=0){//exit libbsapi
        syslog(LOG_ERR,"Unable to terminate libbsapi.");
    }
#endif

    if((rc=pam_start(service,user,&pamconv,&pamh))!=PAM_SUCCESS){
        syslog(LOG_ERR,"Error PAM_START: %s.",pam_strerror(pamh,rc));
        return;
    }
    connect(line,SIGNAL(returnPressed()),this,SLOT(haveResult()));
    line->setText("");
    if((rc=pam_authenticate(pamh,0))!=PAM_SUCCESS){
        message=string("Fingerprint authentication failed.");
    }
    else{
        message=string("Authentication successful.");
    }
    disconnect(line,SIGNAL(returnPressed()),this,SLOT(haveResult()));
    if((rc=pam_end(pamh,rc))!=PAM_SUCCESS){
        syslog(LOG_ERR,"Error PAM_END: %s.",pam_strerror(pamh,rc));
    }

// Grab the fingerprint device so we can use it again    
    if(fp_init()!=0){//init libfprint
        syslog(LOG_ERR,"Unable to init libfprint.");
        return;
    }
#ifdef HAS_LIBBSAPI
    if(ABSInitialize()!=0){//init libbsapi
        syslog(LOG_ERR,"Unable to init libbsapi.");
        return;
    }
#endif
    line->setText(message.data());
    syslog(LOG_DEBUG,"Message: %s.",message.data());
}

void PamTester::haveResult(){
    syslog(LOG_DEBUG,"Caught result.");
    if(pamResponse){
        const char* s=line->text().toLocal8Bit();
        if(strlen(s)==0)
            stopTester=true;
        pamResponse->resp=strdup(s);
        line->setText("");
    }
    returnPressed=true;
}
