/*
 * Project "fingerprintGUI": Services for fingerprint authentication on Linux
 * Module: fingerprintGUI.cpp
 * Purpose: Main for fingerprintGUI
 *
 * @author  Wolfgang Ullrich
 * Copyright (C) 2008-2009  Wolfgang Ullrich
 */

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string>
#include <iostream>
#include <fstream>
#include <QtGui>

#include "ui_fingerprintGUI.h"
#include "MainWindowImpl.h"
#include "DeviceHandler.h"

using namespace std;

bool debugTest=false;
string syslogIdent=string("fingerprintGUI");

int main(int argc, char *argv[]){
    QApplication app(argc, argv);

    openlog(syslogIdent.data(),LOG_NDELAY|LOG_PID,LOG_AUTH);
    setlogmask(LOG_UPTO(LOG_ERR));
    for(int i=0;i<argc;i++){
        if((strcmp(argv[i],"debug")==0)
                | (strcmp(argv[i],"--debug")==0)
                | (strcmp(argv[i],"-d")==0)){
            setlogmask(-1);
            debugTest=true;
        }
    }
    syslog(LOG_INFO,"Started.");
    
    //init libfprint
    if(fp_init()!=0){
        syslog(LOG_ERR,"Unable to init libfprint.");
        return 0;
    }
#ifdef HAS_LIBBSAPI
    //init libbsapi
    if(ABSInitialize()!=0){
        syslog(LOG_ERR,"Unable to init libbsapi.");
        return 0;
    }
#endif
    
    MainWindowImpl mainWindow;
    mainWindow.deviceHandler->rescan();

    mainWindow.show();
    int rc=app.exec();
    
    //exit libfprint
    fp_exit();
    
#ifdef HAS_LIBBSAPI
    //exit libbsapi
    if(ABSTerminate()!=0){
        syslog(LOG_ERR,"Unable to terminate libbsapi.");
    }
#endif
    return rc;
 }
