/*
 * Project "fingerprintGUI": Services for fingerprint authentication on Linux
 * Module: fingerprintPlugin.cpp, fingerprintPlugin.h
 * Purpose: Plugin to be used for authentication with gnome-screensaver and other plugin-aware applications
 *
 * @author  Wolfgang Ullrich
 * Copyright (C) 2008-2009  Wolfgang Ullrich
 */

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <syslog.h>
#include <string>

#include "../include/fingerprintPlugin.h"
#include "ui_fingerprintPlugin.h"

using namespace std;

static string syslogIdent=string("fingerprintPlugin");

int main(int argc,const char **argv){
    mkfifo(PLUGIN_FIFO,0666);   // Create a named pipe to receive messages from fingerprintHelper
    chmod(PLUGIN_FIFO,0666);
    openlog(syslogIdent.data(),LOG_NDELAY|LOG_PID,LOG_AUTH);
    setlogmask(LOG_UPTO(LOG_ERR));
    for(int i=0;i<argc;i++){
        if((strcmp(argv[i],"debug")==0)
                | (strcmp(argv[i],"--debug")==0)
                | (strcmp(argv[i],"-d")==0)){
            setlogmask(-1);
            syslog(LOG_INFO,"Got \"debug\" argument.");
        }
    }
    syslog(LOG_DEBUG,"started.");
    QApplication app(argc,(char **)argv);

    PluginDialog plugin;
    syslog(LOG_DEBUG,"PluginDialog created.");

    WId id=plugin.winId();      // Write window-ID to stdout
    syslog(LOG_DEBUG,"WId: %li sent to stdout.",id);
    fprintf(stdout, "%li\n", id);
    fflush(stdout);

    app.exec();
    unlink(PLUGIN_FIFO);
    syslog(LOG_DEBUG,"QApplication ended.");
    return EXIT_SUCCESS;
}

FifoReader::FifoReader(QLabel *la){
    label=la;   // The QLabel to put text to received from named pipe
}

void FifoReader::run(){
    QFont font;

    if((fifo=open(PLUGIN_FIFO,O_RDONLY))<0){    // This blocks until someone has opened the other end for write
        syslog(LOG_ERR,"Error open fifo: %s",strerror(errno));
        return;
    }
    syslog(LOG_DEBUG,"Fifo opened: %d",fifo);
    char line[1024];
    while(true){
        int rc=read(fifo,line,1024);
        if(rc>0){
            line[rc-1]='\0';
            syslog(LOG_DEBUG,"Read from fifo: %s.",line);
            string s=string(line);
            if(s.find(MSG_STOP)!=string::npos){
                syslog(LOG_DEBUG,"Received STOP message.");
                qApp->exit(EXIT_SUCCESS);
                return;
            }
            if(s.find(MSG_BOLD)!=string::npos){
                font.setPointSize(10);
                font.setBold(true);
                label->setFont(font);
                rc=strlen(MSG_BOLD);
            }
            if(s.find(MSG_NORMAL)!=string::npos){
                font.setPointSize(9);
                font.setBold(false);
                label->setFont(font);
                rc=strlen(MSG_NORMAL);
            }
            label->setText((s.substr(rc,string::npos)).data());    // Show message on statusBar
        }
    }
}

PluginDialog::PluginDialog()
    : QFrame(){
    QMovie *movie;
    setupUi(this);
    UNDECORATED_MODAL_WINDOW_CENTERED
    ANIMATION_MOVIE    
    show();
    fifoReader=new FifoReader(statusBar);
    fifoReader->start();
}
